#pragma implementation
#include <stdio.h>
#include "menucomp.h"
#include "wincomp.h"

CheckedMenuItem::CheckedMenuItem(const char *s) {
  checked = false;
  setLabel(s);
}

CheckedMenuItem::~CheckedMenuItem() { }

MenuItemComponent::MenuItemComponent() {
  enable();
}

MenuItemComponent::~MenuItemComponent() { }

void MenuBarComponent::show(bool vis) {
  if (vis)
    menu->popupAt(Point(0,0),this);
  else {
    menu->setfocus(0);
    menu->hide();
  }
}

void MenuComponent::show(bool vis) {
  if (vis)
    menu->popupAt(origin,this);
  else {
    menu->setfocus(0);
    menu->hide();
  }
}

void MenuComponent::doit(Karet *app) {
  menu->popupAt(origin,app);
}

void CheckedMenuItem::setLabel(const char *s) {
  label = ' ';
  if (s)
    label += s;
  if (isSep())
    disable();
  setChecked(checked);
}

bool CheckedMenuItem::isSep() const {
  return label.length() == 2 && label.chars()[1] == '-';
}

void CheckedMenuItem::setChecked(bool b) {
  checked = b;
  label[0] = b ? 0xFB : ' ';
  MenuItem::operator=(label.chars());
}

void CheckedMenuItem::draw(Window *w,const Rect &r) {
  if (isSep()) {
    w->fill(r,A_NORMAL + ACS_HLINE);
    return;
  }
  MenuItem::draw(w,r);
}

void MenuItemComponent::remoteMethod(int cmd) {
  switch (cmd) {
  case SETSTATE:
    setChecked(readShort() != 0);
    return;
  case Component::DISABLE:
    state(MenuItem::GRAY);
    return;
  case Component::ENABLE:
    enable();
    return;
  case SETLABEL:
    setLabel(readUTF());
    return;
  }
  Component::remoteMethod(cmd);
}

static CheckedMenuItem sepItem("-");

void MenuComponent::remoteMethod(int cmd) {
  Component *obj;
  switch (cmd) {
  case SHOWPOPUP: {
    obj = toolkit->componentAt(readShort());
    int x = descale(readShort(),toolkit->scalex);
    int y = descale(readShort(),toolkit->scaley);
    origin = obj->absLocation() + Point(y,x);
    show(true);
    return;
  }
  case ADDITEM: {
    obj = toolkit->componentAt(readShort());
    if (indexOf(obj) < 0)
      addItem(obj->asMenuItem());
    return;
  }
  case DELITEM:
    delItem(readShort());
    return;
  case ADDSEP:
    addItem(&sepItem);
    return;
  case NEWMENU:
    obj = new MenuComponent;
    toolkit->addComponent(obj,readShort());
    addItem(obj->asMenuItem());
    return;
  case NEWMENUITEM:
  case NEWCHECKMENUITEM:
    obj = new MenuItemComponent;
    toolkit->addComponent(obj,readShort());
    addItem(obj->asMenuItem());
    return;
  }
  MenuItemComponent::remoteMethod(cmd);
}

MenuComponent::MenuComponent(): MenuContainer(new PopupMenu), origin(9,9) { }

MenuComponent::~MenuComponent() { }

MenuBarComponent::MenuBarComponent(): MenuContainer(new BarMenu) { }

MenuBarComponent::~MenuBarComponent() { }

MenuContainer::MenuContainer(Menu *m) {
  menu = m;
}

MenuContainer::~MenuContainer() {
  for (int i = menu->count(); i > 0;)
    menu->remove(--i);
  delete menu;
}

void MenuContainer::addItem(MenuItem *m) {
  if (m)
    (*menu) + m;
}

int MenuContainer::indexOf(Component *c) {
  if (!c) return -1;
  int n = menu->count();
  MenuItem *m = c->asMenuItem();
  if (!m) return -1;
  for (int i = 0; i < n; ++i) {
    if ((*menu)[i] == m)
      return i;		// item already in menu
  }
  return -1;
}

void MenuContainer::delItem(int i) {
  menu->remove(i);
}

void MenuBarComponent::remoteMethod(int cmd) {
  switch (cmd) {
  case NEWMENU: {
    Component *obj = new MenuComponent;
    toolkit->addComponent(obj,readShort());
    addItem(obj->asMenuItem());
    return;
  }
  case ADDHELPMENU:
  case ADDMENU:
    addItem(toolkit->componentAt(readShort())->asMenuItem());
    return;
  case DELMENU:
    delItem(readShort());
    return;
  }
  Component::remoteMethod(cmd);
}

void MenuItemComponent::doit(Karet *) {
  if (state() == MenuItem::ENABLE) {
    Component *par = getParent();
    if (par) {
      par->show(false);
    }
    toolkit->callMethod(compID,MENUPICK);
  }
}

void MenuComponent::draw(Window *w,const Rect &r) {
  origin = w->loc.origin() + r.origin() + Point(1,0);
  MenuItem::draw(w,r);
}
