#pragma interface
#include "component.h"
#include "textedit.h"

class LabelComponent: public Component {
  enum { LEFT, CENTER, RIGHT };
  String txt;
  char alignment;
public:
  LabelComponent();
  void remoteMethod(int cmd);
  virtual void setText(const String &);
  const String &getText() const { return txt; }
  void setAlign(int a);
  void paint(DrawingContext *);
  void getfocus();
};


class Button: public LabelComponent {
public:
  Button();
  void key(int ky);
  void getfocus();
};

class CheckBox: public Button {
  bool checked;
public:
  CheckBox();
  void remoteMethod(int cmd);
  void setState(bool state);
  void getfocus();
  void paint(DrawingContext *);
};

class TextComponent: public TextEdit {
  TextComponent(const TextComponent &);
  operator=(const TextComponent &);
protected:
  int columns;		// restrict input to column width if > 0
public:
  TextComponent();
  void remoteMethod(int cmd);
  virtual const String getText() const = 0;
  virtual void setText(String) = 0;
  void setEditable(bool);
  void key(int);
  void getfocus();
  ~TextComponent();
};

class TextField: public TextComponent {
  TextField(const TextField &);
  operator=(const TextField &);
public:
  TextField() { }
  void remoteMethod(int cmd);
  const String getText() const;
  void setText(String);
  void move(int r,int c);
  void key(int);
#if 0
  void paint(DrawingContext *);
  Dimension size() const;
  void setCursor(int x,int y);
#endif
};

class TextArea: public TextComponent {
  TextArea(const TextArea &);
  operator=(const TextArea &);
public:
  TextArea() { }
  const String getText() const;
  void append(const String &);
  void setText(String);
  void insertText(const String &txt,int pos) { replaceText(txt,pos,pos); }
  void remoteMethod(int cmd);
};
