#ifndef __textdraw_h
#define __textdraw_h
#pragma interface
#include <Display.h>

class DrawingContext {
protected:
  Rect cliprect;
  short translatex, translatey;	// translation
public:
  DrawingContext(const Rect &r);
  Rect clipRect() const;
  virtual class IndirectContext create();
  void clipRect(const Rect &r);
  void translate(int dx, int dy) { translatex += dx; translatey += dy; }
  virtual void writeAttr(int x,int y,int n,const CHTYPE *a) = 0;
  virtual void copyArea(const Rect &,int dx,int dy) = 0;
  virtual void fillRect(const Rect &,CHTYPE attr) = 0;
  virtual CHTYPE get(int x,int y) const;
  class String asString() const;
  void writeStr(int x,int y,const class String &s,CHTYPE attr);
  void writeStr(int x,int y,int n,const class String &s,CHTYPE attr);
  enum LDtype { LD_SINGLE, LD_DOUBLE };
  void drawRect(int x,int y,int w,int h,CHTYPE a,LDtype type);
  virtual ~DrawingContext() { }
};

class IndirectContext: public DrawingContext {
  DrawingContext *dc;
public:
  IndirectContext(DrawingContext *);
  class IndirectContext create();
  void setClip(const Rect &);
  CHTYPE get(int x,int y) const;
  void writeAttr(int x,int y,int n,const CHTYPE *a);
  void copyArea(const Rect &,int dx,int dy);
  void fillRect(const Rect &,CHTYPE attr);
};

class TextImage: public DrawingContext {
  CHTYPE *data;
  unsigned *d;
  short imgwidth, imgheight;
public:
  CHTYPE *rowdata(int i) { return data + d[i]; }
  const CHTYPE *rowdata(int i) const { return data + d[i]; }
  TextImage(int width, int height);
  void writeAttr(int x,int y,int n,const CHTYPE *a);
  void copyArea(const Rect &,int dx,int dy);
  void fillRect(const Rect &,CHTYPE attr);
  void resize(int width, int height);
  CHTYPE get(int x,int y) const;
  ~TextImage();
};
#endif
