// $Log: tuipeer.cc,v $
// Revision 1.7  1997/09/18  21:15:34  stuart
// Forward parameters from EDX environment
//
// Revision 1.6  1997/04/28  20:06:11  stuart
// fix keyword
//
// Revision 1.5  1997/04/28  20:04:54  stuart
// update help
//
// Revision 1.4  1997/04/28  20:01:37  stuart
// Autoload, localtabs, scale
//
// Revision 1.3  1996/12/23  21:35:13  stuart
// split source
//
// Revision 1.2  1996/10/17  13:02:33  stuart
// Menus supported
//
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <Signal.h>
#include <String.h>
#include <Keyboard.h>
#include <Frame.h>
#include <Keydata.h>
#include <socket.h>
#include "remotepeer.h"
#include "toolkit.h"

enum { RUNCLASS = 500, SETPARM = 126, SETPROP = 127 };

static void usage() {
  fputs("$Id: tuipeer.cc,v 1.7 1997/09/18 21:15:34 stuart Exp stuart $\n\
Usage:	tuipeer [-a] [-t[logfile]] [-sw,h] [-l] [-pport] [package.Class ...]\n\
	-a	autoloaded by java, use /dev/tty for screen & keyboard\n\
	-l	local tabs, handle tabs locally instead of through AWT\n\
	-t	trace AWT Thin Client messages\n\
	-s	pretend character cells are w x h pixels\n\
	-p	AWT host:port to connect to, default localhost:9712\n\
	Class	load classes or applets
",stderr);
  exit(1);
}

int main(int argc,char **argv) {
  extern const char version[];
  int port = 9712;
  String host("localhost");
  int sx = 1, sy = 1;
  int i;
  bool trace = false;
  bool localtabs = false;
  bool login = true;
  if (argc < 2)
    usage();
  for (i = 1; i < argc; ++i) {
    if (strncmp(argv[i],"-p",2) == 0) {		// port
      String s(argv[i] + 2);
      if (s.length() == 0)
	s = argv[++i];
      String a[2];
      int n = split(s,a,2,':');
      if (n < 2) {
	if (isdigit(s[0]))
	  port = atoi(s);
	else
	  host = s;
      }
      else {
	host = a[0];
	port = atoi(a[1]);
      }
    }
    else if (strncmp(argv[i],"-t",2) == 0) {	// trace
      trace = true;
      if (argv[i][2])
	freopen(argv[i]+2,"w",stderr);
      //PStrace(stderr);
    }
    else if (strcmp(argv[i],"-l") == 0) {	// local tabs
      localtabs = true;
    }
    else if (strncmp(argv[i],"-s",2) == 0) {	// scale
      String s[2];
      split(argv[i]+2,s,2,',');
      sx = atoi(s[0]);
      if (sx <= 0) sx = 1;
      sy = atoi(s[1]);
      if (sy <= 0) sy = 1;
    }
    else if (strcmp(argv[i],"-a") == 0) {	// autoloaded
      close(0);
      close(1);
      dup(open("/dev/tty",O_RDWR));
      login = false;
    }
    else 
      break;
  }
  //malloc_trace("tuipeer.tr");
  Socket sock;
  if (sock.connect(host,port)) {
    perror(host);
    return 1;
  }
  int fd = sock.getfd();
  Toolkit kit(fd);
  if (login) {
    kit.writeUTF((const char *)getlogin());
    kit.writeUTF("bmsi.tui.TUIKit");
  }
  kit.trace = trace;
  kit.localtabs = localtabs;
  char buf[1024];
  Keydata parm(buf,sizeof buf);
  int plen = parm.parmget();
  if (plen > 0)
    kit.callMethod(0,(int)SETPARM,buf,plen);
  const char *btasdir = getenv("BTASDIR");
  if (btasdir != 0) {
    kit.callMethod(0,(int)SETPROP,"bmsi.btasdir");
    kit.writeUTF(btasdir);
  }
  const char *edxterm = getenv("EDXTERM");
  if (edxterm != 0) {
    kit.callMethod(0,(int)SETPROP,"bmsi.term");
    kit.writeUTF(edxterm);
  }
  const char *logtty = getenv("LOGTTY");
  if (logtty != 0) {
    kit.callMethod(0,(int)SETPROP,"bmsi.tty");
    kit.writeUTF(logtty);
  }
  kit.init(sx,sy);
  Keyboard kbd(stdin);
  Kbegin(1);
  Kdefine("\017",KEY_OPTIONS);
  signal(SIGPIPE,SIG_IGN);
  Signal sigint(SIGINT), sigterm(SIGTERM), sigpipe(SIGPIPE);
  Banner banner(12,version,
		"Copyright 1996,1997 Business Management Systems, Inc.");
  Ksync();
  for (;i < argc; ++i)
    kit.callMethod(0,(int)RUNCLASS,String(argv[i]));
  kit.run();
  Kend();
  return 0;
}
